/*
btrevers.c Version 0.2.0  Btree sweep database in
reverse sequence
Copyright (C) 2003-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Export the database in reverse sequence */
/* Note: the end of line character defines */
/* the end of the record */
/* btrevers output may be used for */
/* a mass update in reverse physical sequence */

#include <bthash.h>

/* recursive routine */
void btrevers(bthdr,ofst)
btreefmt *bthdr;
off_t ofst;
   {
   int i;
   bt_nodefmt *node;
   blkfmt *blk;
   unsigned char *p,*q;
   bthdr->reason = 0;
   blk = (blkfmt *) malloc(bthdr->blksz + 16);
   if (blk == NULL)
      {
      fprintf(stderr,"btrevers: out of memory\n");
      bthdr->reason = BTREE_ERR + 1601;
      return;
      } /* out of mem */
   bttravgt(bthdr,ofst,blk);
   if (bthdr->reason)
      {
      free(blk);
      return;
      } /* travget err */
   if (blk->right)
      {
      btrevers(bthdr,blk->right);
      if (bthdr->reason)
	 {
	 free(blk);
	 return;
	 } /* if recursion err */
      } /* if right */
   p = (unsigned char *) blk
      + bthdr->blkhdr
      + (bthdr->rcdnode * blk->count)
      - bthdr->rcdnode;
   node = (bt_nodefmt *) p;
   i = blk->count;
   while (i--)
      {
      p = (unsigned char *) node + bthdr->rcdhdr;
      if (*(p + bthdr->dletofst) != DELETED)
	 {
         printf("%5d. %08x %5d ",
	    ++bthdr->seq,
	    (unsigned int) ofst,
	    i + 1);
         q = (unsigned char *) p + bthdr->rcdlen;
         while (p < q) putchar(*p++);
         putchar('\n');
	 } /* if not deleted index node */
      if (node->left)
	 {
	 btrevers(bthdr,node->left);
         if (bthdr->reason)
            {
            free(blk);
            return;
            } /* if recursion err */
	 } /* if left */
      p = (unsigned char *) node - bthdr->rcdnode;
      node = (bt_nodefmt *) p;
      } /* left */
   free(blk);
   } /* btrevers */
